/**
 * Active Campaign Controls
 *
 */

/**
 * Imports
 */
import Select from 'react-select';
import { addQueryArgs } from '@wordpress/url';
import apiFetch from '@wordpress/api-fetch';
import { getFormFields } from '../../';

/**
 * Internal block libraries
 */
import { __ } from '@wordpress/i18n';
import {
	Fragment,
	useEffect,
	useMemo,
	useState,
} from '@wordpress/element';
import {
	TextControl,
	Button,
	Spinner,
	ToggleControl,
	SelectControl,
	ExternalLink,
} from '@wordpress/components';
import { BasePanelBody } from '@base/components';

const HELP_URL = 'https://avanam.org';

/**
 * Build the Measure controls
 * @returns {object} Measure settings.
 */
function ActiveCampaignOptions( { formInnerBlocks, parentClientId, settings, save } ) {

	const [ api, setApi ] = useState( '' );
	const [ isSavedApi, setIsSavedApi ] = useState( false );
	const [ apiBase, setApiBase ] = useState( '' );
	const [ isSavedApiBase, setIsSavedApiBase ] = useState( false );
	const [ isSaving, setIsSaving ] = useState( false );
	const [ lists, setLists ] = useState( false );
	const [ isFetching, setIsFetching ] = useState( false );
	const [ listsLoaded, setListsLoaded ] = useState( false );
	const [ isFetchingAttributes, setIsFetchingAttributes ] = useState( false );
	const [ listAttr, setListAttr ] = useState( false );
	const [ listAttrLoaded, setListAttrLoaded ] = useState( false );
	const [ isFetchingGroups, setIsFetchingAutomations ] = useState( false );
	const [ automations, setAutomations ] = useState( false );
	const [ automationLoaded, setAutomationsLoaded ] = useState( false );
	const [ isFetchingTags, setIsFetchingTags ] = useState( false );
	const [ tags, setTags ] = useState( false );
	const [ tagsLoaded, setTagsLoaded ] = useState( false );

	useEffect( () => {
		apiFetch( {
			path: '/wp/v2/settings',
			method: 'GET',
		} ).then( ( response ) => {
			setApi( response.base_blocks_activecampaign_api_key );
			setApiBase( response.base_blocks_activecampaign_api_base );

			if ( '' !== response.base_blocks_activecampaign_api_key && '' !== response.base_blocks_activecampaign_api_base ) {
				setIsSavedApi( true );
				setIsSavedApiBase( true );
			}
		});

	}, [] );

	const fields = useMemo( () => getFormFields( formInnerBlocks ), [ parentClientId ] );

	const saveMap = ( value, uniqueID ) => {
		let updatedMap = { ...settings.map }
		updatedMap[uniqueID] = value;
		save( { map: updatedMap } );
	};

	const getLists = () => {
		if ( !api ) {
			setLists( [] );
			setListsLoaded( true );
			return;
		}

		setIsFetching( true );

		apiFetch( {
			path: addQueryArgs(
				'/bb-activecampaign/v1/get',
				{ endpoint: 'lists' },
			),
		} )
			.then( ( lists ) => {
				const theLists = [];
				lists.map( ( item ) => {
					theLists.push( { value: item.id, label: item.name } );
				} );

				setLists( theLists );
				setListsLoaded( true );
				setIsFetching( false );
			} )
			.catch( ( err ) => {
				setLists( [] );
				setListsLoaded( true );
				setIsFetching( false );
			} );
	};

	const getAutomations = () => {
		setIsFetchingAutomations( true );

		apiFetch( {
			path: addQueryArgs(
				'/bb-activecampaign/v1/get',
				{ endpoint: 'automations' },
			),
		} )
			.then( ( automations ) => {
				const theAutomations = [];
				automations.map( ( item ) => {
					theAutomations.push( { value: item.id, label: item.name } );
				} );

				setAutomations( theAutomations );
				setAutomationsLoaded( true );
				setIsFetchingAutomations( false );
			} )
			.catch( (err) => {
				setAutomations( [] );
				setAutomationsLoaded( true );
				setIsFetchingAutomations( false );
			} );
	};

	const getTags = () => {
		setIsFetchingTags( true );

		apiFetch( {
			path: addQueryArgs(
				'/bb-activecampaign/v1/get',
				{ endpoint: 'tags' },
			),
		} )
			.then( ( tags ) => {
				const theTags = [];
				if ( tags ) {
					tags.map( ( item ) => {
						theTags.push( { value: item.id, label: item.tag } );
					} );
				}

				setTags( theTags );
				setTagsLoaded( true );
				setIsFetchingTags( false );
			} )
			.catch( () => {
				setTags( [] );
				setTagsLoaded( true );
				setIsFetchingTags( false );
			} );
	};

	const getAttributes = () => {
		setIsFetchingAttributes( true );

		apiFetch( {
			path: addQueryArgs(
				'/bb-activecampaign/v1/get',
				{ endpoint: 'fields' },
			),
		} )
			.then( ( list ) => {
				const theAttributes = [];
				theAttributes.push( { value: null, label: 'None' } );
				theAttributes.push( { value: 'email', label: __('Email', 'gutenam-blocks' ) + ' *' } );
				theAttributes.push( { value: 'firstName', label: __('First Name', 'gutenam-blocks' ) } );
				theAttributes.push( { value: 'lastName', label: __( 'Last Name', 'gutenam-blocks' ) } );
				theAttributes.push( { value: 'phone', label: __('Phone', 'gutenam-blocks' ) } );

				list.fields.map( ( item, index ) => {
					theAttributes.push( { value: item.id, label: item.title } );
				} );

				setListAttr( theAttributes );
				setListAttrLoaded( true );
				setIsFetchingAttributes( false );
			} )
			.catch( () => {
				const theAttributes = [];
				theAttributes.push( { value: null, label: 'None' } );
				theAttributes.push( { value: 'email', label: __('Email', 'gutenam-blocks' ) + ' *' } );
				theAttributes.push( { value: 'firstName', label: __('First Name', 'gutenam-blocks' ) } );
				theAttributes.push( { value: 'lastName', label: __( 'Last Name', 'gutenam-blocks' ) } );
				theAttributes.push( { value: 'phone', label: __('Phone', 'gutenam-blocks' ) } );

				setListAttr( theAttributes );
				setListAttrLoaded( true );
				setIsFetchingAttributes( false );
			} );
	};

	const removeAPI = () => {
		setApi( '' );
		setApiBase( '' );
		setIsSaving( true );

		const settingModel = new wp.api.models.Settings( {
			base_blocks_activecampaign_api_key : '',
			base_blocks_activecampaign_api_base: '',
		} );
		settingModel.save().then( () => {
			setIsSavedApi( false );
			setIsSavedApiBase( false );
			setIsSaving( false );
		} );

	};

	const saveAPI = () => {
		setIsSaving( true );

		const settingModel = new wp.api.models.Settings( {
			base_blocks_activecampaign_api_key : api,
			base_blocks_activecampaign_api_base: apiBase,
		} );
		settingModel.save().then( response => {
			setIsSaving( false );
			setIsSavedApi( true );
			setIsSavedApiBase( true );
		} );
	};

	const hasLists = Array.isArray( lists ) && lists.length > 0;
	const hasAttr = Array.isArray( listAttr ) && listAttr.length > 0;
	const hasAutomations = Array.isArray( automations ) && automations.length > 0;
	const hasTags = Array.isArray( tags ) && tags.length > 0;

	return (
		<BasePanelBody
			title={__( 'ActiveCampaign Settings', 'gutenam-blocks' )}
			initialOpen={false}
			panelName={'bb-activecampaign-settings'}
		>
			<p>
				<ExternalLink href={HELP_URL}>{__( 'Get help', 'gutenam-blocks' )}</ExternalLink>
			</p>
			<TextControl
				label={__( 'API Key', 'gutenam-blocks' )}
				value={api}
				onChange={value => setApi( value )}
			/>
			<TextControl
				label={__( 'API URL', 'gutenam-blocks' )}
				placeholder={'https://youaccount.api-us1.com'}
				value={apiBase}
				onChange={value => setApiBase( value )}
			/>
			<div className="components-base-control">
				<Button
					isPrimary
					onClick={() => saveAPI()}
					disabled={'' === api}
				>
					{isSaving ? __( 'Saving', 'gutenam-blocks' ) : __( 'Save', 'gutenam-blocks' )}
				</Button>
				{api !== '' && (
					<Fragment>
						&nbsp;
						<Button
							isSecondary
							onClick={() => removeAPI()}
						>
							{__( 'Remove', 'gutenam-blocks' )}
						</Button>
					</Fragment>
				)}
			</div>
			{isSavedApi && isSavedApiBase && (
				<Fragment>
					{isFetching && (
						<Spinner/>
					)}
					{!isFetching && !hasLists && (
						<Fragment>
							<h2 className="bt-heading-size-title">{__( 'Select List', 'gutenam-blocks' )}</h2>
							{( !listsLoaded ? getLists() : '' )}
							{!Array.isArray( lists ) ?
								<Spinner/> :
								__( 'No Lists found.', 'gutenam-blocks' )}
						</Fragment>

					)}
					{!isFetching && hasLists && (
						<Fragment>
							<h2 className="bt-heading-size-title">{__( 'Select List', 'gutenam-blocks' )}</h2>
							<Select
								value={( undefined !== settings && undefined !== settings && undefined !== settings.list ? settings.list : '' )}
								onChange={( value ) => {
									save( { list: ( value ? value : {} ) } );
								}}
								id={'mc-list-selection'}
								isClearable={true}
								options={lists}
								isMulti={false}
								maxMenuHeight={200}
								placeholder={__( 'Select List' )}
							/>

							<Fragment>
								{isFetchingGroups && (
									<Spinner/>
								)}
								{!isFetchingGroups && !hasAutomations && (
									<Fragment>
										<h2 className="bt-heading-size-title">{__( 'Select Automation', 'gutenam-blocks' )}</h2>
										{( !automationLoaded ? getAutomations() : '' )}
										{!Array.isArray( automations ) ?
											<Spinner/> :
											__( 'No Groups found.', 'gutenam-blocks' )}
									</Fragment>
								)}
								{ ! isFetchingGroups && hasAutomations && (
									<Fragment>
										<h2 className="bt-heading-size-title">{__( 'Select Automation', 'gutenam-blocks' )}</h2>
										<Select
											value={( undefined !== settings && undefined !== settings && undefined !== settings.automation ? settings.automation : '' )}
											onChange={( value ) => {
												save( { automation: ( value ? value : [] ) } );
											}}
											id={'mc-automation-selection'}
											isClearable={true}
											options={automations}
											maxMenuHeight={200}
											placeholder={__( 'Select Automation' )}
										/>
									</Fragment>
								)}
								{isFetchingTags && (
									<Spinner/>
								)}
								{!isFetchingTags && !hasTags && (
									<Fragment>
										<h2 className="bt-heading-size-title">{__( 'Select Tags', 'gutenam-blocks' )}</h2>
										{( !tagsLoaded ? getTags() : '' )}
										{!Array.isArray( tags ) ?
											<Spinner/> :
											__( 'No Tags found.', 'gutenam-blocks' )}
									</Fragment>

								)}
								{!isFetchingTags && hasTags && (
									<Fragment>
										<h2 className="bt-heading-size-title">{__( 'Select Tags', 'gutenam-blocks' )}</h2>
										<Select
											value={( undefined !== settings && undefined !== settings && undefined !== settings.tags ? settings.tags : '' )}
											onChange={( value ) => {
												save( { tags: ( value ? value : [] ) } );
											}}
											id={'mc-tag-selection'}
											isClearable={true}
											options={tags}
											isMulti={true}
											maxMenuHeight={200}
											placeholder={__( 'Select Tags' )}
										/>
									</Fragment>
								)}
								{isFetchingAttributes && (
									<Spinner/>
								)}
								{!isFetchingAttributes && !hasAttr && (
									<Fragment>
										<h2 className="bt-heading-size-title">{__( 'Map Fields', 'gutenam-blocks' )}</h2>
										{( !listAttrLoaded ? getAttributes() : '' )}
										{!Array.isArray( listAttr ) ?
											<Spinner/> :
											__( 'No Fields found.', 'gutenam-blocks' )}
									</Fragment>

								)}
								{!isFetchingAttributes && hasAttr && (
									<Fragment>
										<h2 className="bt-heading-size-title">{__( 'Map Fields', 'gutenam-blocks' )}</h2>
										{fields && (
											fields.map( ( item, index ) => {
												return (
													<div key={index} className="bb-field-map-item">
														<div className="bb-field-map-item-form">
															<p className="bb-field-map-item-label">{__( 'Form Field', 'gutenam-blocks' )}</p>
															<p className="bb-field-map-item-name">{item.label}</p>
														</div>
														<SelectControl
															label={__( 'Select Field:' )}
															options={listAttr}
															value={( undefined !== settings.map && undefined !== settings.map[ item.uniqueID ] && settings.map[ item.uniqueID ] ? settings.map[ item.uniqueID ] : '' )}
															onChange={( value ) => {
																saveMap( value, item.uniqueID );
															}}
														/>
													</div>
												);
											} )
										)}
									</Fragment>
								)}
							</Fragment>
						</Fragment>
					)}
				</Fragment>
			)}
		</BasePanelBody>
	);
}

export default ( ActiveCampaignOptions );
