/**
 * BLOCK: Base Advanced Form
 */

/**
 * Import Css
 */
import './editor.scss';
/**
 * External dependencies
 */
import classnames from 'classnames';

/**
 * Internal block libraries
 */
import { __ } from '@wordpress/i18n';
import { isEmpty } from 'lodash';
import { useSelect, useDispatch } from '@wordpress/data';
import { store as editorStore } from '@wordpress/editor';
import { formBlockIcon, formTemplateContactIcon } from '@base/icons';
import {
	BasePanelBody,
} from '@base/components';
import { useBlockProps, InspectorControls } from '@wordpress/block-editor';
import {
	Placeholder,
	Spinner,
} from '@wordpress/components';
import {
	store as coreStore,
	EntityProvider,
} from '@wordpress/core-data';

import { useEntityAutoDraft } from './hooks';
import { SelectOrCreatePlaceholder, SelectForm } from './components';
import { getUniqueId, getPostOrFseId } from '@base/helpers';


/**
 * Internal dependencies
 */
import EditInner from './edit-inner';
import { useEffect } from '@wordpress/element';

export function Edit( props ) {

	const {
		attributes,
		setAttributes,
		clientId,
	} = props;

	const { id, uniqueID } = attributes;

	const blockClasses = classnames( {
		'wp-block-base-advanced-form'                : true,
		[ `wp-block-base-advanced-form${uniqueID}` ] : uniqueID,
	} );
	const blockProps = useBlockProps( {
		className: blockClasses
	} );
	const { post, currentPostType, postId } = useSelect(
		( select ) => {
			return {
				post: id && select( coreStore ).getEditedEntityRecord( 'postType', 'base_form', id ),
				currentPostType: select( editorStore ).getCurrentPostType(),
				postId: select( editorStore ).getCurrentPostId(),
			}
		},
		[ id ],
	);

	const { addUniqueID } = useDispatch( 'baseblocks/data' );
	const { isUniqueID, isUniqueBlock, parentData, isPreviewMode } = useSelect(
		( select ) => {
			return {
				isUniqueID: ( value ) => select( 'baseblocks/data' ).isUniqueID( value ),
				isUniqueBlock: ( value, clientId ) => select( 'baseblocks/data' ).isUniqueBlock( value, clientId ),
				isPreviewMode: select( 'core/block-editor' ).getSettings().__unstableIsPreviewMode,
				parentData: {
					rootBlock: select( 'core/block-editor' ).getBlock( select( 'core/block-editor' ).getBlockHierarchyRootClientId( clientId ) ),
					postId: select( 'core/editor' ).getCurrentPostId(),
					reusableParent: select('core/block-editor').getBlockAttributes( select('core/block-editor').getBlockParentsByBlockName( clientId, 'core/block' ).slice(-1)[0] ),
					editedPostId: select( 'core/edit-site' ) ? select( 'core/edit-site' ).getEditedPostId() : false
				}
			};
		},
		[ clientId ]
	);

	if( isPreviewMode ) {
		return(
			<>
				{formTemplateContactIcon}
			</>
		);
	}

	useEffect( () => {
		const postOrFseId = getPostOrFseId( props, parentData );
		let uniqueId = getUniqueId( uniqueID, clientId, isUniqueID, isUniqueBlock, postOrFseId );
		if ( uniqueId !== uniqueID ) {
			attributes.uniqueID = uniqueId;
			setAttributes( { uniqueID: uniqueId } );
			addUniqueID( uniqueId, clientId );
		} else {
			addUniqueID( uniqueId, clientId );
		}
		if ( currentPostType === 'base_form' ) {
			// Lame workaround for gutenberg to prevent showing the block Validity error.
			window.wp.data.dispatch( 'core/block-editor' ).setTemplateValidity( true );
		}
	}, [] );

	{/* Directly editing from via base_form post type */}
	if ( currentPostType === 'base_form' ) {
		return (
			<div {...blockProps}>
				<EditInner {...props} direct={true} id={ postId }/>
			</div>
		);
	}
	return (
		<div {...blockProps}>
			{/* No form selected, display chooser */}
			{ id === 0 && (
				<Chooser
					id={id}
					post={post}
					commit={( nextId ) => setAttributes( { id: nextId } )}
				/>
			)}

			{/* Form selected but not loaded yet, show spinner */}
			{id > 0 && isEmpty( post ) && (
				<>
					<Placeholder
						className="bb-select-or-create-placeholder"
						label={__( 'Base Form', 'gutenam-blocks' )}
						icon={ formBlockIcon }
					>
						<Spinner/>
					</Placeholder>
					<InspectorControls>
						<BasePanelBody
								panelName={'bb-advanced-form-selected-switch'}
								title={ __( 'Selected Form', 'gutenam-blocks' ) }
							>
							<SelectForm
								postType="base_form"
								label={__( 'Selected Form', 'gutenam-blocks' )}
								hideLabelFromVision={ true }
								onChange={ ( nextId ) => {
									setAttributes( { id: parseInt( nextId ) } ) 
								} }
								value={ id }
							/>
						</BasePanelBody>
					</InspectorControls>
				</>
			)}
			{id > 0 && !isEmpty( post ) && ( isEmpty(post.content ) && isEmpty( post.blocks ) ) && (
				<Chooser
					id={id}
					post={post}
					commit={( nextId ) => setAttributes( { id: parseInt( nextId ) } )}
				/>
			)}
			{id > 0 && !isEmpty( post ) && post.status === "trash" && (
				<>
					<Placeholder
						className="bb-select-or-create-placeholder"
						label={__( 'Base Form', 'gutenam-blocks' )}
						icon={ formBlockIcon }
					>
						{ __( 'The selected from is in the trash.', 'gutenam-blocks' ) }
					</Placeholder>
					<InspectorControls>
						<BasePanelBody
								panelName={'bb-advanced-form-selected-switch'}
								title={ __( 'Selected Form', 'gutenam-blocks' ) }
							>
							<SelectForm
								postType="base_form"
								label={__( 'Selected Form', 'gutenam-blocks' )}
								hideLabelFromVision={ true }
								onChange={ ( nextId ) => {
									setAttributes( { id: parseInt( nextId ) } ) 
								} }
								value={ id }
							/>
						</BasePanelBody>
					</InspectorControls>
				</>
			)}

			{/* Form selected and loaded, display form */}
			{id > 0 && !isEmpty( post ) && post.status !== 'trash' && ( ! isEmpty( post.content ) || !isEmpty (post.blocks ) ) && (
				<EntityProvider kind="postType" type="base_form" id={id}>
					<EditInner {...props} direct={false} id={id}/>
				</EntityProvider>
			)}
		</div>
	);
}

export default ( Edit );

function Chooser( { id, post, commit } ) {
	const [ isAdding, addNew ] = useEntityAutoDraft( 'base_form', 'base_form' );
	const onAdd = async () => {
		try {
			const response = await addNew();
			commit( response.id );
		} catch ( error ) {
			console.error( error );
		}
	};

	return (
		<SelectOrCreatePlaceholder
			postType="base_form"
			label={__( 'Advanced Form', 'gutenam-blocks' )}
			instructions={__( 'Select an existing form or create a new one.', 'gutenam-blocks' )}
			placeholder={__( 'Select Form', 'gutenam-blocks' )}
			onSelect={commit}
			isSelecting={id && isEmpty( post )}
			onAdd={onAdd}
			isAdding={isAdding}
		/>
	);
}
