/**
 * Internal dependencies
 */
import FormFieldLabel from '../../label';

/**
 * WordPress dependencies
 */
import { TextControl, TextareaControl, SelectControl, ToggleControl, CheckboxControl } from '@wordpress/components';
import { __ } from '@wordpress/i18n';
import { InspectorControls, useBlockProps } from '@wordpress/block-editor';
import { useSelect, useDispatch } from '@wordpress/data';
import { BasePanelBody, InspectorControlTabs, ResponsiveRangeControls, SelectParentBlock } from '@base/components';
import { useEffect, useState } from '@wordpress/element';
import { without } from 'lodash';
import {
	getUniqueId,
	getPreviewSize,
} from '@base/helpers';
import classNames from 'classnames';
import { DuplicateField, FieldBlockAppender, FieldName } from '../../components';

function FieldFile( { attributes, setAttributes, isSelected, clientId, context, name } ) {
	const { uniqueID, required, label, showLabel, maxSizeMb, allowedTypes, helpText, ariaDescription, maxWidth, maxWidthUnit, minWidth, minWidthUnit, inputName, requiredMessage } = attributes;

	const wpMaxUploadSizeBytes = base_blocks_params.wp_max_upload_size;
	const wpMaxUploadSizeMb = formatBytesToMb( wpMaxUploadSizeBytes );
	const wpMaxUploadSizePretty = formatBytes( wpMaxUploadSizeBytes );

	const [ activeTab, setActiveTab ] = useState( 'general' );
	const { addUniqueID } = useDispatch( 'baseblocks/data' );
	const { isUniqueID, isUniqueBlock, previewDevice } = useSelect(
		( select ) => {
			return {
				isUniqueID   : ( value ) => select( 'baseblocks/data' ).isUniqueID( value ),
				isUniqueBlock: ( value, clientId ) => select( 'baseblocks/data' ).isUniqueBlock( value, clientId ),
				previewDevice: select( 'baseblocks/data' ).getPreviewDeviceType(),
			};
		},
		[ clientId ],
	);

	useEffect( () => {
		let uniqueId = getUniqueId( uniqueID, clientId, isUniqueID, isUniqueBlock );
		setAttributes( { uniqueID: uniqueId } );
		addUniqueID( uniqueId, clientId );
	}, [] );
	useEffect( () => {
		if ( maxSizeMb > wpMaxUploadSizeMb ) {
			setAttributes( { maxSizeMb: wpMaxUploadSizeMb } );
		}
	}, [wpMaxUploadSizeMb] );
	const previewMaxWidth = getPreviewSize( previewDevice, ( maxWidth && maxWidth[ 0 ] ? maxWidth[ 0 ] : '' ), ( maxWidth && maxWidth[ 1 ] ? maxWidth[ 1 ] : '' ), ( maxWidth && maxWidth[ 2 ] ? maxWidth[ 2 ] : '' ) );
	const previewMinWidth = getPreviewSize( previewDevice, ( minWidth && minWidth[ 0 ] ? minWidth[ 0 ] : '' ), ( minWidth && minWidth[ 1 ] ? minWidth[ 1 ] : '' ), ( minWidth && minWidth[ 2 ] ? minWidth[ 2 ] : '' ) );
	const classes = classNames( {
		'bb-adv-form-field': true,
		'bb-adv-form-infield-type-input': true,
	} );
	const blockProps = useBlockProps( {
		className: classes,
		style    : {
			maxWidth: '' !== previewMaxWidth ? previewMaxWidth + ( maxWidthUnit ? maxWidthUnit : '%' ) : undefined,
			minWidth: '' !== previewMinWidth ? previewMinWidth + ( minWidthUnit ? minWidthUnit : 'px' ) : undefined,
		},
	} );

	const getSizeOptions = () => {
		const sizeOptions = [];

		for ( let i = 1; ( i * 5 ) <= Math.min( 25, wpMaxUploadSizeMb ); i++ ) {
			sizeOptions.push( {
				value: ( i * 5 ),
				label: ( i * 5 ) + ' MB',
			} );
		}
		return sizeOptions;
	};

	const toggleAllowedTypes = ( type ) => {
		let newTypes = [];

		if ( allowedTypes.includes( type ) ) {
			newTypes = without( allowedTypes, type );
		} else {
			newTypes = [ ...allowedTypes, type ];
		}

		setAttributes( { allowedTypes: newTypes } );
	};

	return (
		<>
			<style>
				{isSelected && (
					<>
						{`.block-editor-block-popover__inbetween-container .block-editor-block-list__insertion-point.is-with-inserter { display: none }`};
					</>
				)}
			</style>
			<div {...blockProps}>
				<DuplicateField
					clientId={clientId}
					name={name}
					attributes={attributes}
				/>
				<InspectorControls>
					<SelectParentBlock
						label={ __( 'View Form Settings', 'gutenam-blocks' ) }
						clientId={ clientId }
						parentSlug={ 'base/advanced-form' }
					/>
					<InspectorControlTabs
						panelName={'advanced-form-text-general'}
						setActiveTab={ ( value ) => setActiveTab( value ) }
						activeTab={ activeTab }
						allowedTabs={ [ 'general', 'advanced' ] }
					/>


					{ ( activeTab === 'general' ) &&
						<>
							<BasePanelBody
								title={__( 'Field Controls', 'gutenam-blocks' )}
								initialOpen={true}
								panelName={ 'bb-adv-form-text-controls' }
							>
								<ToggleControl
									label={__( 'Required', 'gutenam-blocks' )}
									checked={required}
									onChange={( value ) => setAttributes( { required: value } )}
								/>
								<TextControl
									label={__( 'Field Label', 'gutenam-blocks' )}
									value={label}
									onChange={( value ) => setAttributes( { label: value } )}
								/>
								<ToggleControl
									label={__( 'Show Label', 'gutenam-blocks' )}
									checked={showLabel}
									onChange={( value ) => setAttributes( { showLabel: value } )}
								/>
								<TextareaControl
									label={__( 'Description', 'gutenam-blocks' )}
									help={ __( 'This will be displayed under the input and can be used to provide direction on how the field should be filled out.', 'gutenam-blocks' )}
									value={helpText}
									onChange={( value ) => setAttributes( { helpText: value } )}
								/>
							</BasePanelBody>
							<BasePanelBody
								title={__( 'File Options', 'gutenam-blocks' )}
							>

								<SelectControl
									label={__( 'File Size Limit', 'gutenam-blocks' )}
									value={maxSizeMb}
									onChange={value => {
										setAttributes( { maxSizeMb: parseInt( value ) } );
									}}
									options={getSizeOptions()}
									max={wpMaxUploadSizeMb}
									help={__( 'WordPress max upload size: ', 'gutenam-blocks' ) + ' ' + wpMaxUploadSizePretty}
								/>

								<h2>{__( 'Allowed File Types', 'gutenam-blocks' )}</h2>
								<CheckboxControl
									label={__( 'Images', 'gutenam-blocks' )}
									help="jpeg, jpg, gif, and png"
									checked={allowedTypes.includes( 'image' )}
									onChange={( value ) => toggleAllowedTypes( 'image' )}
								/>

								<CheckboxControl
									label={__( 'PDF', 'gutenam-blocks' )}
									checked={allowedTypes.includes( 'pdf' )}
									onChange={( value ) => toggleAllowedTypes( 'pdf' )}
								/>

								<CheckboxControl
									label={__( 'Audio', 'gutenam-blocks' )}
									help="mp3, wav, ogg, wma, m4a, mid, mka"
									checked={allowedTypes.includes( 'audio' )}
									onChange={( value ) => toggleAllowedTypes( 'audio' )}
								/>

								<CheckboxControl
									label={__( 'Video', 'gutenam-blocks' )}
									help="mp4, mpg, mpeg, mpe, m4v, avi, mov"
									checked={allowedTypes.includes( 'video' )}
									onChange={( value ) => toggleAllowedTypes( 'video' )}
								/>

								<CheckboxControl
									label={__( 'Documents', 'gutenam-blocks' )}
									help="csv, doc, ppt, docx, ody, odp, ods, txt, rtf, xls, xlsx, odt, ott"
									checked={allowedTypes.includes( 'document' )}
									onChange={( value ) => toggleAllowedTypes( 'document' )}
								/>

								<CheckboxControl
									label={__( 'Zip Archive', 'gutenam-blocks' )}
									checked={allowedTypes.includes( 'archive' )}
									onChange={( value ) => toggleAllowedTypes( 'archive' )}
								/>

							</BasePanelBody>
						</>
					}
					{ ( activeTab === 'advanced' ) &&
						<>
							<BasePanelBody
								title={__( 'Field Width', 'gutenam-blocks' )}
								initialOpen={true}
								panelName={ 'bb-adv-form-text-width' }
							>
								<ResponsiveRangeControls
									label={__( 'Max Width', 'gutenam-blocks' )}
									value={( undefined !== maxWidth && undefined !== maxWidth[ 0 ] ? maxWidth[ 0 ] : '' )}
									onChange={value => {
										setAttributes( { maxWidth: [ value, ( undefined !== maxWidth && undefined !== maxWidth[ 1 ] ? maxWidth[ 1 ] : '' ), ( undefined !== maxWidth && undefined !== maxWidth[ 2 ] ? maxWidth[ 2 ] : '' ) ] } );
									}}
									tabletValue={( undefined !== maxWidth && undefined !== maxWidth[ 1 ] ? maxWidth[ 1 ] : '' )}
									onChangeTablet={( value ) => {
										setAttributes( { maxWidth: [ ( undefined !== maxWidth && undefined !== maxWidth[ 0 ] ? maxWidth[ 0 ] : '' ), value, ( undefined !== maxWidth && undefined !== maxWidth[ 2 ] ? maxWidth[ 2 ] : '' ) ] } );
									}}
									mobileValue={( undefined !== maxWidth && undefined !== maxWidth[ 2 ] ? maxWidth[ 2 ] : '' )}
									onChangeMobile={( value ) => {
										setAttributes( { maxWidth: [ ( undefined !== maxWidth && undefined !== maxWidth[ 0 ] ? maxWidth[ 0 ] : '' ), ( undefined !== maxWidth && undefined !== maxWidth[ 1 ] ? maxWidth[ 1 ] : '' ), value ] } );
									}}
									min={0}
									max={( maxWidthUnit === 'px' ? 2000 : 100 )}
									step={1}
									unit={maxWidthUnit ? maxWidthUnit : '%'}
									onUnit={( value ) => {
										setAttributes( { maxWidthUnit: value } );
									}}
									units={[ 'px', '%', 'vw' ]}
								/>
								<ResponsiveRangeControls
									label={__( 'Min Width', 'gutenam-blocks' )}
									value={( undefined !== minWidth && undefined !== minWidth[ 0 ] ? minWidth[ 0 ] : '' )}
									onChange={value => {
										setAttributes( { minWidth: [ value, ( undefined !== minWidth && undefined !== minWidth[ 1 ] ? minWidth[ 1 ] : '' ), ( undefined !== minWidth && undefined !== minWidth[ 2 ] ? minWidth[ 2 ] : '' ) ] } );
									}}
									tabletValue={( undefined !== minWidth && undefined !== minWidth[ 1 ] ? minWidth[ 1 ] : '' )}
									onChangeTablet={( value ) => {
										setAttributes( { minWidth: [ ( undefined !== minWidth && undefined !== minWidth[ 0 ] ? minWidth[ 0 ] : '' ), value, ( undefined !== minWidth && undefined !== minWidth[ 2 ] ? minWidth[ 2 ] : '' ) ] } );
									}}
									mobileValue={( undefined !== minWidth && undefined !== minWidth[ 2 ] ? minWidth[ 2 ] : '' )}
									onChangeMobile={( value ) => {
										setAttributes( { minWidth: [ ( undefined !== minWidth && undefined !== minWidth[ 0 ] ? minWidth[ 0 ] : '' ), ( undefined !== minWidth && undefined !== minWidth[ 1 ] ? minWidth[ 1 ] : '' ), value ] } );
									}}
									min={0}
									max={( minWidthUnit === 'px' ? 2000 : 100 )}
									step={1}
									unit={minWidthUnit ? minWidthUnit : 'px'}
									onUnit={( value ) => {
										setAttributes( { minWidthUnit: value } );
									}}
									units={[ 'px', '%', 'vw' ]}
								/>
							</BasePanelBody>
							<BasePanelBody
								title={__( 'Extra Settings', 'gutenam-blocks' )}
								initialOpen={false}
								panelName={ 'bb-adv-form-text-extra-settings' }
							>
								<FieldName
									value={inputName}
									uniqueID={uniqueID}
									onChange={( value ) => setAttributes( { inputName: value.replace(/[^a-z0-9-_]/gi, '') } ) }
								/>
								<TextControl
									label={__( 'Input aria description', 'gutenam-blocks' )}
									value={ariaDescription}
									onChange={( value ) => setAttributes( { ariaDescription: value } )}
									help={ __( 'This content will be hidden by default and exposed to screen readers as the aria-describedby attribute for this form field. Note that the normal description field will no longer be used for aria-describedby.', 'gutenam-blocks' ) }
								/>
								{ required && (
									<TextControl
										label={__( 'Field error message when required', 'gutenam-blocks' )}
										value={requiredMessage}
										onChange={( value ) => setAttributes( { requiredMessage: value } )}
										placeholder={( undefined !== label ? label : '' ) + ' ' + __( 'is required', 'gutenam-blocks' )}
									/>
								)}
							</BasePanelBody>
						</>
					}

				</InspectorControls>
				<div className={'bb-form-field-container'}>
					<div className={'bb-form-field'}>
						<FormFieldLabel
							required={required}
							label={label}
							showLabel={showLabel}
							setAttributes={setAttributes}
							isSelected={isSelected}
							name={name}
						/>
						<input type={'file'} />

						{helpText && <span className="bb-adv-form-help">{helpText}</span>}
					</div>
				</div>
				<FieldBlockAppender inline={true} className="bb-custom-inbetween-inserter" getRoot={clientId}/>
			</div>
		</>
	);
}

export default FieldFile;

function formatBytes( bytes ) {
	if ( bytes === 0 ) {
		return '0 Bytes';
	}

	const k = 1024;
	const sizes = [ 'Bytes', 'KB', 'MB', 'GB', 'TB' ];

	const i = Math.floor( Math.log( bytes ) / Math.log( k ) );

	return parseFloat( ( bytes / Math.pow( k, i ) ).toFixed( 0 ) ) + ' ' + sizes[ i ];
}

function formatBytesToMb( bytes ) {
	if ( bytes === 0 ) {
		return 0;
	}

	const k = 1024;

	return parseFloat( ( bytes / Math.pow( k, 2 ) ).toFixed( 0 ) );
}
