/**
 * External dependencies
 */
import AceEditor from "react-ace";
import "ace-builds/src-noconflict/mode-css";
import 'ace-builds/src-noconflict/snippets/css';
import "ace-builds/src-noconflict/theme-textmate";
import { assign } from 'lodash';
 /**
 * Import WordPress Internals
 */
  import {
	useState,
 } from '@wordpress/element';
import {
	InspectorControls,
} from '@wordpress/block-editor';
import { store as blocksStore, hasBlockSupport } from '@wordpress/blocks';
import {
	PanelBody,
	Button,
	Modal,
} from '@wordpress/components';
import { useSelect } from '@wordpress/data';
import { createHigherOrderComponent } from '@wordpress/compose';
import {
	addFilter,
} from '@wordpress/hooks';
import { __, sprintf } from '@wordpress/i18n';
import {
	showSettings,
} from '@base/helpers';
 /**
 * Add Block CSS attributes
 *
 * @param {array} settings The block settings.
 * @returns {array} The block settings with block css added.
 */
export function blockCSSAttribute( settings, name ) {
	if ( hasBlockSupport( settings, 'bbcss' ) ) {
		settings.attributes = assign( settings.attributes, {
			baseBlockCSS: {
				type: 'string',
				default: '',
			},
		} );
	}
	return settings;
}
addFilter( 'blocks.registerBlockType', 'base/blockCSS', blockCSSAttribute );

/**
 * Build the block css control
 */
 const BlockCSSComponent = createHigherOrderComponent( ( BlockEdit ) => {
	return ( props ) => {
		if ( ! props.isSelected ) {
			return <BlockEdit { ...props } />;
		}
		const { hasBlockCSS, openTab } = useSelect(
			( select ) => {
				const hasSupport = select( blocksStore ).hasBlockSupport( props.name, 'bbcss' );
				let openTab = 'general';
				if ( hasSupport ) {
					openTab = typeof select( 'baseblocks/data' ).getOpenSidebarTabKey === "function" ? select( 'baseblocks/data' ).getOpenSidebarTabKey( props.name.replace('base/', '') + select( 'core/block-editor' ).getSelectedBlockClientId(), 'general' ) : 'advanced';
				}
				return {
					hasBlockCSS: hasSupport,
					openTab: openTab,
				};
			},
			[]
		);
		const [ isOpen, setOpen ] = useState( false );
		const openModal = () => setOpen( true );
		const closeModal = () => setOpen( false );
		if ( hasBlockCSS && openTab == 'advanced' && showSettings( 'show', 'base/customcss' ) ) {
			const { attributes: { baseBlockCSS }, setAttributes } = props;
			const customCSSEditor = (
				<>
					<AceEditor
						mode="css"
						theme="textmate"
						onLoad={ ( editor ) => {
							editor.renderer.setScrollMargin( 16, 16, 16, 16 );
							editor.renderer.setPadding( 16 );
						} }
						onChange={ ( value ) => {
							if ( value !== 'selector {\n\n}' ) {
								setAttributes( {
									baseBlockCSS: value,
								} );
							}
						} }
						showPrintMargin={false}
						highlightActiveLine={ false }
						showGutter={true}
						fontSize={12}
						value={ baseBlockCSS || 'selector {\n\n}' }
						maxLines={ undefined }
						minLines={ undefined }
						width="100%"
						height="calc( 100% - 50px)"
						setOptions={ {
							enableBasicAutocompletion: true,
							enableLiveAutocompletion: true,
							enableSnippets: true,
							showLineNumbers: true,
							tabSize: 2,
						} }
					/>
					{ /* translators: The %s is for selector code */ }
					<p dangerouslySetInnerHTML={ {
						__html: sprintf(
							/* translators: The %s is for selector code */
							__( 'Use %s rule to change block styles.', 'gutenam-blocks' ),
							'<code>selector</code>'
						)
						} } />
				</>
			);
			return (
				<>
					<BlockEdit { ...props } />
					<InspectorControls>
						<PanelBody
							title={ __( 'Custom CSS', 'gutenam-blocks' ) }
							initialOpen={ false }
						>
							<>
								<Button className={'base-css-modal-open'} icon={'editor-expand'} onClick={ openModal }>
									{ __( 'Edit in Modal', 'gutenam-blocks' ) }
								</Button>
								{ isOpen && (
									<Modal className={'base-css-modal'} title={ __( 'Custom CSS', 'gutenam-blocks' ) } onRequestClose={ closeModal }>
										{ customCSSEditor }
										<Button variant="secondary" onClick={ closeModal }>
											{ __( 'Close', 'gutenam-blocks' ) }
										</Button>
									</Modal>
								) }
								<AceEditor
									mode="css"
									theme="textmate"
									onLoad={ ( editor ) => {
										editor.renderer.setScrollMargin( 16, 16, 16, 16 );
										editor.renderer.setPadding( 16 );
									} }
									onChange={ ( value ) => {
										if ( value !== 'selector {\n\n}' ) {
											setAttributes( {
												baseBlockCSS: value,
											} );
										}
									} }
									showPrintMargin={false}
									highlightActiveLine={ false }
									showGutter={true}
									fontSize={12}
									value={ baseBlockCSS || 'selector {\n\n}' }
									maxLines={ 20 }
									minLines={ 5 }
									width="100%"
									height="300px"
									setOptions={ {
										enableBasicAutocompletion: true,
										enableLiveAutocompletion: true,
										enableSnippets: true,
										showLineNumbers: true,
										tabSize: 2,
									} }
								/>
								<p style={ { marginBottom: 20 } } />
								
								{ /* translators: The %s is for selector code */ }
								<p dangerouslySetInnerHTML={ {
									__html: sprintf(
										/* translators: The %s is for selector code */
										__( 'Use %s rule to change block styles.', 'gutenam-blocks' ),
										'<code>selector</code>'
									)
								 } } />
							</>
						</PanelBody>
					</InspectorControls>
				</>
			);
		}
		return <BlockEdit { ...props } />;
	};
}, 'BlockCSSComponent' );
addFilter( 'editor.BlockEdit', 'base/blockCSSControls', BlockCSSComponent );
